#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

import json
import pathlib
from typing import List

import jsons
import numpy as np
import pandas as pd

from rbfoptgo.common import Cost, ParameterValue
from rbfoptgo.client import Client
from rbfoptgo.config import Config
from rbfoptgo.report import Report
from rbfoptgo import names


class Evaluator:
    __config: Config
    __client: Client
    __parameter_names: List[str]
    __evaluations: []
    __root_dir: pathlib.Path
    __report: Report
    __iterations: int

    def __init__(self, config: Config, client: Client, parameter_names: List[str], root_dir: pathlib.Path):
        self.__config = config
        self.__client = client
        self.__parameter_names = parameter_names
        self.__evaluations = []
        self.__root_dir = root_dir
        self.__iterations = 0

    def __np_array_to_parameter_values(self, raw_values: np.ndarray) -> List[ParameterValue]:
        parameter_values = []
        for i, raw_value in enumerate(raw_values):
            parameter_values.append(
                ParameterValue(name=self.__parameter_names[i], value=int(raw_value)),
            )
        return parameter_values

    def estimate_cost(self, raw_values: np.ndarray) -> Cost:
        self.__iterations += 1

        parameter_values = self.__np_array_to_parameter_values(raw_values)
        cost, invalid_parameter_combination = self.__client.estimate_cost(parameter_values)

        # store evaluation result for the future use
        entry = {name: value for (name, value) in zip(self.__parameter_names, raw_values)}
        entry[names.Iteration] = self.__iterations
        entry[names.Cost] = cost
        entry[names.InvalidParameterCombination] = invalid_parameter_combination

        self.__evaluations.append(entry)

        return cost

    def register_report(
            self,
            cost: Cost,
            optimum: np.ndarray,
            iterations: int,
            evaluations: int,
            fast_evaluations: int,
    ):
        report = Report(
            bounds=self.__config.rbfopt.parameters,
            cost=cost,
            optimum=self.__np_array_to_parameter_values(optimum),
            iterations=iterations,
            evaluations=evaluations,
            fast_evaluations=fast_evaluations,
        )

        self.__client.register_report(report)
        self.__report = report

    def dump(self) -> (pd.DataFrame, Report):
        # dump history of evaluations for future usage
        evaluations = pd.DataFrame(self.__evaluations)
        file_path = self.__root_dir.joinpath("evaluations.csv")
        evaluations.to_csv(file_path, header=True, index=False)

        # dump report for future usage
        file_path = self.__root_dir.joinpath("report.json")
        self.__report.save_to_file(file_path)

        return evaluations, self.__report


#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

import functools
import typing

import matplotlib.axes
import matplotlib.image
import matplotlib.pyplot as plt
import matplotlib.ticker
import numpy as np
import pandas as pd
import scipy.interpolate
import scipy.stats
from adjustText import adjust_text
from colorhash import ColorHash

from rbfoptgo import names
from rbfoptgo.config import Config, InvalidParameterCombinationRenderPolicy
from rbfoptgo.report import Report


class Renderer:
    __df: pd.DataFrame
    __report: Report
    __config: Config
    __transparent: bool

    def __init__(self, config: Config, df: pd.DataFrame, report: Report, transparent: bool = False):
        self.__df = df.loc[:, df.columns != names.Iteration]
        self.__report = report
        self.__config = config
        self.__transparent = transparent

    def __prepare_df(self, policy: InvalidParameterCombinationRenderPolicy) -> pd.DataFrame:
        match policy:
            case InvalidParameterCombinationRenderPolicy.omit:
                # Filter values corresponding to ErrInvalidParameterCombination params
                return self.__df[self.__df.invalid_parameter_combination == False]
            case InvalidParameterCombinationRenderPolicy.assign_closest_valid_value:
                # avoid white spots on the heatmap
                unique_costs = self.__df.cost.unique()
                unique_costs.sort()
                # -1 stands for ErrInvalidParameterCombinationCost, take -2 - the next closest
                closest_valid_value = unique_costs[-2]

                # make df with replaced values
                df = self.__df.copy()
                mask = df.invalid_parameter_combination == True
                df.cost.where(~mask, closest_valid_value, inplace=True)
                return df
            case _:
                raise ValueError(f"unknown policy: {policy}")

    @property
    @functools.cache
    def __parameter_column_names(self) -> typing.List[str]:
        utility_columns = (names.Cost, names.InvalidParameterCombination)
        return list(filter(lambda x: x not in utility_columns, self.__df.columns))

    def __cost_bounds(self, df: pd.DataFrame) -> typing.List[str]:
        cost = df[names.Cost]
        return [cost.min(), cost.max()]

    def scatterplots(self):
        self.__render_scatterplot_group(only_optimal_values=False)
        self.__render_scatterplot_group(only_optimal_values=True)

    def __render_scatterplot_group(self, only_optimal_values: bool):
        column_names = self.__parameter_column_names

        if len(column_names) <= 2:
            n_rows, n_columns = 1, len(column_names)
        else:
            n_columns = 2
            if len(column_names) % 2 == 0:
                n_rows = int(len(column_names) / n_columns)
            else:
                n_rows = int(len(column_names) / n_columns) + 1

        # this multiplier is purely empirical...
        figsize = (6 * n_rows, 6 * n_columns)

        fig, axes = plt.subplots(nrows=n_rows, ncols=n_columns, figsize=figsize,
                                 squeeze=False, constrained_layout=True)
        axes = axes.flat

        for i in range(len(axes)):
            if i < len(column_names):
                self.__render_scatterplot(axes[i], column_names[i], only_optimal_values=only_optimal_values)
            else:
                axes[i].axis('off')

        fig.tight_layout()

        suffix = "only_optimal_values" if only_optimal_values else "all_values"
        figure_path = self.__config.root_dir.joinpath(f"scatterplot_{suffix}.png")
        fig.savefig(figure_path, transparent=self.__transparent)

    def __render_scatterplot(self, ax: matplotlib.axes.Axes, col_name: str, only_optimal_values: bool):
        df = self.__prepare_df(policy=self.__config.plot.scatter_plot_policy)
        data = pd.DataFrame({col_name: df[col_name], names.Cost: df[names.Cost]})

        if only_optimal_values:
            # for every argument value, pick the best cost function value
            data = data.groupby(col_name)[names.Cost].agg(lambda x: x.min()).reset_index()

        color = ColorHash(col_name).hex
        ax.plot(data[col_name], data[names.Cost], linewidth=0, marker='o', color=color)

        ax.set_xlabel(col_name, fontsize=14)
        ax.set_ylabel('Cost function', fontsize=14)

        # draw point with optimum
        opt_arg = self.__report.optimum_argument(col_name)
        opt_val = self.__report.cost
        ax.scatter(opt_arg, opt_val, color='red', marker='o', s=100)
        ax.annotate("{:.2f}".format(opt_val), (opt_arg, opt_val))

        # set equal limits
        (cost_min, cost_max) = self.__cost_bounds(df)
        # ax.set_ybound(lower=cost_min, upper=cost_max)
        ax.set_ylim(bottom=cost_min, top=cost_max)

    def heatmaps(self):
        df = self.__prepare_df(policy=self.__config.plot.heatmap_render_policy)

        # TODO: one can pass a particular set of interpolation methods,
        #  but honestly I can't see any significant difference between them.
        # methods = ['none', 'nearest', 'bilinear', 'bicubic', 'spline16',
        #            'spline36', 'hanning', 'hamming', 'hermite', 'kaiser', 'quadric',
        #            'catrom', 'gaussian', 'bessel', 'mitchell', 'sinc', 'lanczos']
        methods = ['hamming']

        for method in methods:
            print(f"rendering heatmap matrix using method {method}")
            self.__pairwise_heatmaps(df=df, interpolation=method)
            self.__pairwise_heatmap_matrix(df=df, interpolation=method)

    def __pairwise_heatmaps(self, df: pd.DataFrame, interpolation: str):
        column_names = self.__parameter_column_names
        for i in range(len(column_names) - 1):
            for j in range(i + 1, len(column_names)):
                self.__pairwise_heatmap(df=df,
                                        col_name_1=column_names[i],
                                        col_name_2=column_names[j],
                                        interpolation=interpolation)

    def __pairwise_heatmap(self,
                           df: pd.DataFrame,
                           col_name_1: str,
                           col_name_2: str,
                           interpolation: str,
                           ):

        figsize = (6, 6)

        fig, ax = plt.subplots(figsize=figsize, constrained_layout=True)

        im = self.__pairwise_heatmap_interpolate(
            df=df,
            ax=ax,
            col_name_1=col_name_1,
            col_name_2=col_name_2,
            interpolation=interpolation,
            x_label=True,
            y_label=True,
        )

        fig.colorbar(im, ax=ax, shrink=0.6)

        figure_path = self.__config.root_dir.joinpath(f"heatmap_{col_name_1}_{col_name_2}_{interpolation}.png")
        fig.savefig(figure_path, transparent=self.__transparent)

    def __pairwise_heatmap_matrix(self, df: pd.DataFrame, interpolation: str):
        column_names = self.__parameter_column_names

        # approximate size that make image look well
        figsize = (3 * len(column_names), 3 * len(column_names))

        fig, axes = plt.subplots(nrows=len(column_names) - 1,
                                 ncols=len(column_names) - 1,
                                 figsize=figsize,
                                 constrained_layout=True,
                                 )
        im = None
        for i in range(len(column_names) - 1):
            for j in range(0, i):
                axes[j, i].axis('off')
            for j in range(i + 1, len(column_names)):
                col_name_1, col_name_2 = column_names[i], column_names[j]
                ax = axes[j - 1, i]
                im = self.__pairwise_heatmap_interpolate(
                    df=df,
                    ax=ax,
                    col_name_1=col_name_1,
                    col_name_2=col_name_2,
                    interpolation=interpolation,
                    x_label=j == len(column_names) - 1,
                    y_label=i == 0,
                )

        cbar = fig.colorbar(im, ax=axes, shrink=0.6)
        cbar.ax.tick_params(labelsize=24)

        figure_path = self.__config.root_dir.joinpath(f"heatmap_matrix_{interpolation}.png")
        fig.savefig(figure_path, transparent=self.__transparent, dpi=300)

    def __pairwise_heatmap_interpolate(self,
                                       df: pd.DataFrame,
                                       ax: matplotlib.axes.Axes,
                                       col_name_1: str,
                                       col_name_2: str,
                                       interpolation: str,
                                       x_label: bool,
                                       y_label: bool,
                                       ) -> matplotlib.image.AxesImage:
        data = df[[col_name_1, col_name_2, names.Cost]]

        # select the minimums
        data = data.groupby([col_name_1, col_name_2])[names.Cost].agg(lambda x: x.min()).reset_index()

        # check if data is sufficient
        if data.shape[0] < 4:
            raise ValueError('Too little data to render grid, try to increase number of iterations')

        # compute grid bounds
        xs0 = data[col_name_1]
        ys0 = data[col_name_2]
        x_min, x_max = xs0.min(), xs0.max()
        y_min, y_max = ys0.min(), ys0.max()
        extent = [x_min, x_max, y_min, y_max]
        N = 100j
        xs, ys = np.mgrid[x_min:x_max:N, y_min:y_max:N]

        zs0 = data[names.Cost]

        # interpolate data
        resampled = scipy.interpolate.griddata(
            points=(xs0, ys0),
            values=zs0,
            xi=(xs, ys),
            method='cubic',
        )

        # render interpolated grid
        (cost_min, cost_max) = self.__cost_bounds(df)
        im = ax.imshow(
            resampled.T,
            cmap='jet',
            origin='lower',
            interpolation=interpolation,
            vmin=cost_min,
            vmax=cost_max,
            extent=extent,
        )

        # draw point with optimum
        opt_x, opt_y, opt_val = self.__derive_optimum_coordinates(col_name_1, col_name_2, )

        t = ax.text(opt_x, opt_y, opt_val, ha='center', va='center', fontsize=20)
        t.set_bbox(dict(facecolor='white', alpha=0.75, edgecolor='red'))

        adjust_text(
            texts=[t],
            ax=ax,
            arrowprops=dict(arrowstyle='->', color='red'),
            expand_text=(2, 2),
            expand_align=(2, 2),
            expand_objects=(2, 2),
            expand_points=(2, 2),
        )
        ax.scatter(opt_x, opt_y, color='red', marker='o', s=100)

        # borders
        x_gap = (x_max - x_min) * 0.025
        y_gap = (y_max - y_min) * 0.025
        ax.set_xlim(left=x_min-x_gap, right=x_max+x_gap)
        ax.set_ylim(bottom=y_min-y_gap, top=y_max+y_gap)

        # assign axes labels
        ax.tick_params(axis='x', which='major', labelsize=16)
        ax.tick_params(axis='y', which='major', labelsize=16)
        if x_label:
            ax.set_xlabel(col_name_1, fontsize=18)
        if y_label:
            ax.set_ylabel(col_name_2, fontsize=18)

        return im

    def __derive_optimum_coordinates(self,
                                     col_name_1: str, col_name_2: str,
                                     ) -> (float, float, float):
        col_val_1 = self.__report.optimum_argument(col_name_1)
        col_val_2 = self.__report.optimum_argument(col_name_2)
        cost_val = self.__report.cost

        # There must be a much better way to achieve that, PRs are welcome
        if cost_val > 100:
            cost_val = int(cost_val)
        if cost_val > 10:
            cost_val = np.around(cost_val, 1)
        if cost_val > 1:
            cost_val = np.around(cost_val, 2)
        if cost_val > 0.1:
            cost_val = np.around(cost_val, 3)
        if cost_val > 0.01:
            cost_val = np.around(cost_val, 4)
        if cost_val > 0.001:
            cost_val = np.around(cost_val, 5)

        return col_val_1, col_val_2, cost_val

    def radar(self):
        # convert optimum values to df
        df = pd.DataFrame(self.__report.optimum).T
        df.columns = df.iloc[0]
        df.drop(df.index[0], inplace=True)

        # explanation: https://www.python-graph-gallery.com/390-basic-radar-chart
        values = df.iloc[0].values.flatten().tolist()
        values += values[:1]

        N = len(df.iloc[0])
        angles = [n / float(N) * 2 * np.pi for n in range(N)]
        angles += angles[:1]

        ax = plt.subplot(111, polar=True)
        ax.set_xticks(angles[:-1], df.columns, size=14)

        ax.set_rlabel_position(0)

        ax.plot(angles, values, linewidth=1, linestyle='solid')
        ax.fill(angles, values, 'b', alpha=0.1)

        plt.savefig(self.__config.root_dir.joinpath('polar.png'), transparent=self.__transparent)



#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

import json
import os
from dataclasses import dataclass
from typing import List

import jsons

from rbfoptgo.common import Cost, ParameterValue
from rbfoptgo.config import Parameter


@dataclass
class Report:
    bounds: List[Parameter]
    optimum: List[ParameterValue]
    cost: Cost
    iterations: int
    evaluations: int
    fast_evaluations: int

    def optimum_argument(self, name: str) -> int:
        for pv in self.optimum:
            if pv.name == name:
                return pv.value

        raise ValueError(f"unexpected name {name}")

    def save_to_file(self, file_path: os.PathLike):
        with open(file_path, "w") as f:
            obj = jsons.dump(self)
            json.dump(obj, f)

    @classmethod
    def load_from_file(cls, file_path: os.PathLike) -> 'Report':
        with open(file_path, "r") as f:
            data = json.load(f)
            return jsons.load(data, cls)



#!/usr/bin/env python

#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

import pathlib

import pandas as pd
from matplotlib import rc

from rbfoptgo.config import Config
from rbfoptgo.plot import Renderer
from rbfoptgo.report import Report

rc('font', **{'family': 'serif', 'serif': ['Roboto']})

debug_dir = pathlib.Path('/home/isaev/trouble/optimization/20220908_2330_HDD_SSD_Volume_IO_CPU_262144')


def main():
    config = Config.from_file(debug_dir.joinpath("config.json"))
    evaluations = pd.read_csv(debug_dir.joinpath("evaluations.csv"))
    report = Report.load_from_file(debug_dir.joinpath("report.json"))
    renderer = Renderer(config=config, df=evaluations, report=report, transparent=False)
    renderer.heatmaps()


if __name__ == '__main__':
    main()



#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

from http import HTTPStatus
from typing import List
from urllib.parse import urljoin

import jsons
import requests

from rbfoptgo.common import Cost, ParameterValue
from rbfoptgo.report import Report
import rbfoptgo.names as names


class Client:
    url_head: str
    session: requests.Session

    def __init__(self, endpoint: str):
        self.url_head = f'http://{endpoint}'
        self.session = requests.Session()

    def estimate_cost(self, parameter_values: List[ParameterValue]) -> (Cost, bool):
        print(f"request '{parameter_values}'")

        payload = dict(parameter_values=parameter_values)
        response = self.session.get(
            urljoin(self.url_head, 'estimate_cost'),
            json=jsons.dump(payload),
        )

        print(f"response code={response.status_code} body={response.json()}")

        if response.status_code != HTTPStatus.OK:
            raise ValueError(f'invalid status code {response.status_code}')
        else:
            return response.json()[names.Cost], response.json()[names.InvalidParameterCombination]

    def register_report(self, report: Report):
        print(f"request '{report}'")

        payload = dict(report=report)
        response = self.session.post(
            urljoin(self.url_head, 'register_report'),
            json=jsons.dump(payload),
        )

        print(f"response code={response.status_code} response={response}")

        if response.status_code != HTTPStatus.OK:
            raise ValueError(f'invalid status code {response.status_code}')


#!/usr/bin/env python

#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

import pathlib
import sys

import rbfopt

from rbfoptgo.client import Client
from rbfoptgo.config import Config
from rbfoptgo.evaluator import Evaluator
from rbfoptgo.plot import Renderer


def main():
    # prepare configuration
    root_dir = pathlib.Path(sys.argv[1])
    config_path = root_dir.joinpath("config.json")
    config = Config.from_file(config_path)
    print(f"config: {config}")

    client = Client(config.endpoint)
    evaluator = Evaluator(config=config, client=client, parameter_names=config.rbfopt.var_names, root_dir=root_dir)

    bb = rbfopt.RbfoptUserBlackBox(obj_funct=evaluator.estimate_cost, **config.rbfopt.user_black_box)

    # perform optimization
    rbfopt_settings = rbfopt.RbfoptSettings(**config.rbfopt.settings)

    alg = rbfopt.RbfoptAlgorithm(rbfopt_settings, bb)

    # post report to server
    evaluator.register_report(*alg.optimize())
    evaluations, report = evaluator.dump()

    # render plots
    renderer = Renderer(config, evaluations, report)
    renderer.scatterplots()
    renderer.heatmaps()
    renderer.radar()


if __name__ == "__main__":
    main()



#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE
import json
import os
import pathlib
from dataclasses import dataclass
from datetime import datetime
from time import mktime
from enum import Enum
from typing import List, Any, Dict

import numpy as np


@dataclass
class Bound:
    left: int
    right: int

    @staticmethod
    def from_dict(obj: Any) -> 'Bound':
        _left = int(obj.get("left"))
        _right = int(obj.get("right"))
        return Bound(_left, _right)


@dataclass
class Parameter:
    bound: Bound
    name: str

    @staticmethod
    def from_dict(obj: Any) -> 'Parameter':
        _bound = Bound.from_dict(obj.get("bound"))
        _name = str(obj.get("name"))
        return Parameter(_bound, _name)


class InvalidParameterCombinationRenderPolicy(Enum):
    omit = 1
    assign_closest_valid_value = 2


@dataclass
class PlotConfig:
    scatter_plot_policy: InvalidParameterCombinationRenderPolicy
    heatmap_render_policy: InvalidParameterCombinationRenderPolicy

    @staticmethod
    def from_dict(obj: Any) -> 'PlotConfig':
        _scatter_plot_policy = InvalidParameterCombinationRenderPolicy[str(obj.get("scatter_plot_policy"))]
        _heatmap_render_policy = InvalidParameterCombinationRenderPolicy[str(obj.get("heatmap_render_policy"))]
        return PlotConfig(_scatter_plot_policy, _heatmap_render_policy)


@dataclass
class RBFOptConfig:
    parameters: List[Parameter]
    max_evaluations: int
    max_iterations: int
    init_strategy: str
    invalid_parameter_combination_cost: int

    @staticmethod
    def from_dict(obj: Any) -> 'RBFOptConfig':
        _parameters = [Parameter.from_dict(y) for y in obj.get("parameters")]
        _max_evaluations = int(obj.get("max_evaluations"))
        _max_iterations = int(obj.get("max_iterations"))
        _init_strategy = str(obj.get("init_strategy"))
        _invalid_parameter_combination_cost = int(obj.get("invalid_parameter_combination_cost"))
        return RBFOptConfig(_parameters, _max_evaluations, _max_iterations, _init_strategy,
                            _invalid_parameter_combination_cost)

    @property
    def var_names(self) -> List[str]:
        var_names = []
        for i, param in enumerate(self.parameters):
            var_names.append(param.name)

        return var_names

    @property
    def user_black_box(self) -> Dict:
        dimensions = len(self.parameters)
        var_lower = np.zeros(shape=(dimensions,))
        var_upper = np.zeros(shape=(dimensions,))
        var_types = np.chararray(shape=(dimensions,))

        for i, param in enumerate(self.parameters):
            var_lower[i] = param.bound.left
            var_upper[i] = param.bound.right
            var_types[i] = 'I'

        return dict(
            dimension=dimensions,
            var_lower=var_lower,
            var_upper=var_upper,
            var_type=var_types,
        )

    @property
    def settings(self) -> Dict:
        return dict(
            max_evaluations=self.max_evaluations,
            max_iterations=self.max_iterations,
            rand_seed=int(mktime(datetime.now().timetuple())),
            init_strategy=self.init_strategy,
        )


@dataclass
class Config:
    root_dir: os.PathLike
    endpoint: str
    rbfopt: RBFOptConfig
    plot: PlotConfig

    @staticmethod
    def from_dict(obj: Any) -> 'Config':
        _root_dir = pathlib.Path(str(obj.get("root_dir")))
        _endpoint = str(obj.get("endpoint"))
        _rbfopt = RBFOptConfig.from_dict(obj.get("rbfopt"))
        _plot = PlotConfig.from_dict(obj.get("plot"))
        return Config(_root_dir, _endpoint, _rbfopt, _plot)

    @staticmethod
    def from_file(path: os.PathLike) -> 'Config':
        with open(path, "r") as f:
            data = json.load(f)
            return Config.from_dict(data)

# Example Usage
# jsonstring = json.loads(myjsonstring)
# root = Root.from_dict(jsonstring)



#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

from typing import Final

Cost: Final = "cost"
InvalidParameterCombination: Final = "invalid_parameter_combination"
Iteration: Final = "iteration"



#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

from dataclasses import dataclass

Cost = float


@dataclass
class ParameterValue:
    name: str
    value: int


#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

from setuptools import setup, find_packages
import pathlib

HERE = pathlib.Path(__file__).parent

setup(name='rbfopt-go',
      # version_config={
      #     "dev_template": "{tag}",
      # },
      setuptools_git_versioning={
          "enabled": True,
          "template": "{tag}",
          "dirty_template": "{tag}",
      },
      description='Find better configuration of your Go service with derivative-free optimization algorithms',
      author='Vitaly Isaev',
      author_email='vitalyisaev2@gmail.com',
      url='https://github.com/newcloudtechnologies/rbfopt-go',
      packages=find_packages(),
      setup_requires=["setuptools-git-versioning"],
      install_requires=(
          "Pyomo==6.1.2",
          "adjustText",
          "colorhash==1.0.4",
          "jsons",
          "matplotlib",
          "numpy",
          "pandas",
          "rbfopt==4.2.2",
          "requests",
          "scipy",
          "urllib3",
      ),
      license_file="LICENSE",
      package_dir={
          '': '.'
      },
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Topic :: Scientific/Engineering :: Mathematics",
      ],
      entry_points={
          'console_scripts': [
              'rbfopt-go-wrapper = rbfoptgo.main:main',
          ]
      },
      zip_safe=False,
      python_requires=">=3.7",
      )
